/*
 * Copyright (c) 2005
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author(s): Philipp Scholl <scholl@teco.edu>
 */
package edu.teco.particles.pComponent;

import java.awt.event.ActionListener;
import java.util.ArrayList;

import edu.teco.particles.libParticle.ParticlePacket;
import edu.teco.particles.pComponent.io.DefaultTemplate;
import edu.teco.particles.pComponent.io.ParticlePacketListener;
import edu.teco.particles.pComponent.io.ParticlePacketsEvent;
import edu.teco.particles.pComponent.io.ParticleTemplate;

/**
 * a Multiplexer for Particle Network traffic.
 * 
 * Functionality: - match all registered ParticleTemplate against the current
 * traffic.
 * 
 * This can be used to do something like this: \code ParticleTemplate tmp = new
 * TupleTemplate( "ACL" ); // will be called when the template matched.
 * tmp.addActionListener( new ActionListener() { ... } );
 * 
 * Multiplexer multi = new Multiplexer( 5555 ); multi.addTemplate( tmp );
 * \endcode
 */
public class Multiplexer implements ParticlePacketListener
{
  /**
   * the socket to receive on.
   */
  private SocketThread _socket;

  /**
   * create a new Multiplexer.
   * 
   * @param port
   *          the port to receive packets on.
   */
  public Multiplexer(int port)
  {
    _socket = new SocketThread();
    _socket.setRecvPort(port);
    _socket.addParticlePacketListener(this);

    _templates = new ArrayList<ParticleTemplate>();

    start();
  }

  protected static Multiplexer _this       = null;

  private static long          currentTime = 0;

  public void start()
  {
    _socket.setReceiving(true);
  }

  public void stop()
  {
    _socket.setReceiving(false);
  }

  /**
   * @name TemplateHandling. add/remove templates. @{
   */
  private ArrayList<ParticleTemplate> _templates;

  private DefaultTemplate             _defaultTemplate;

  private DefaultTemplate getDefaultTemplate()
  {
    if (_defaultTemplate == null) _defaultTemplate = new DefaultTemplate();
    return _defaultTemplate;
  }

  public void addTemplate(ParticleTemplate template)
  {
    assert (!(template instanceof DefaultTemplate)) : "use addDefaultListener()";

    synchronized (_templates)
    {
      _templates.add(template);
    }
  }

  /**
   * wipe out the complete (also all childs of a template)
   * 
   * @param template
   */
  public void removeTemplates(ParticleTemplate template)
  {
    synchronized (_templates)
    {
      for (ParticleTemplate p : template.getChilds())
        removeTemplates(p);

      _templates.remove(template);
    }
  }

  public void removeTemplate(ParticleTemplate template)
  {
    synchronized (_templates)
    {
      _templates.remove(template);
    }
  }

  public void clear()
  {
    synchronized (_templates)
    {
      _templates.clear();
      synchronized (getDefaultTemplate().getActionListeners())
      {
        getDefaultTemplate().getActionListeners().clear();
      }
    }
  }

  /** @} */

  /**
   * here we receive packets from the network.
   * 
   * they will be put through all registered Template and if none matched the
   * default template is called, if it's registered.
   */
  public void particlePackets(ParticlePacketsEvent e)
  {
    for (ParticlePacket pck : _socket.getBuffer())
    {
      injectPacket(pck);
    }
  }

  public void injectPacket(ParticlePacket pck)
  {
    currentTime = System.currentTimeMillis();

    synchronized (_templates)
    {
      boolean gotMatch = false;

      for (ParticleTemplate template : _templates)
      {
        if (template.match(pck)) gotMatch = true;
      }

      if (!gotMatch) _fireDefaultMatch(pck);
    }
  }

  private void _fireDefaultMatch(ParticlePacket pck)
  {
    getDefaultTemplate().match(pck);
  }

  public void addDefaultListener(ActionListener listener)
  {
    if (!getDefaultTemplate().getActionListeners().contains(listener))
      getDefaultTemplate().addActionListener(listener);
  }

  public void removeDefaultListener(ActionListener listener)
  {
    getDefaultTemplate().removeActionListener(listener);
  }

  public static long currentTime()
  {
    return currentTime;
  }
}
