/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.util;

import java.util.HashMap;

/**
 * a Factory for TupleParser.
 * 
 * makes a match from String to Parser. Or matches a ParticleContext to a
 * Parser.
 */
public class TupleParserFactory
{
  /** update the function getAvailableTypes also ! */
  public final static String SHORT = "Short (8bit)";

  public final static String LONG  = "Long (16bit)";

  public final static String TIME  = "cts";

  /**
   * returns all available types which can be parsed
   * 
   * @return Object[] containing the availables types as Strings.
   */
  public static String[] getAvailableTypes()
  {
    return new String[] { SHORT, LONG, TIME };
  }

  /**
   * creates a new TupleParser.
   * 
   * @param type
   *          the type that you want be the parser for (example: Long or Short)
   * @return a matching tupleParser
   * @throws IllegalArgumentException
   */
  public static TupleParser create(String type) throws IllegalArgumentException
  {
    if (SHORT.contains(type))
    {
      return new ShortParser();
    }
    if (LONG.contains(type))
    {
      return new LongParser();
    }
    if (TIME.contains(type))
    {
      return new CTSParser();
    }

    throw new IllegalArgumentException("no ParticleParser known for type: "
        + type);
  }

  /** pointer to a context -> parser mapping */
  protected static HashMap<ParticleContext, TupleParser> _contextParserMap;

  /** factory method */
  protected static HashMap<ParticleContext, TupleParser> _getContextMap()
  {
    if (_contextParserMap == null)
    {
      _contextParserMap = new HashMap<ParticleContext, TupleParser>();
    }

    return _contextParserMap;
  }

  /**
   * a factory method for TupleParsers. Creates a default Parser for a
   * ParticleContext if none is available or this context.
   * 
   * currently a Short Unsigned parser is returned by default.
   * 
   * @param context
   *          the context to create the parser for.
   * @return a default parser for this context.
   */
  public static TupleParser getTupleParser(ParticleContext context)
  {
    TupleParser parser = null;

    if (_getContextMap().containsKey(context))
    {
      parser = _getContextMap().get(context);
    }
    else
    {
      parser = create(SHORT);
      _getContextMap().put(context, parser);
    }

    assert (parser != null);
    return parser;
  }

  /**
   * creates a parser parsing as stated in def.
   * 
   * @param ctx
   *          the context to get the parser for.
   * @param def
   *          The defintion of a parsers to look for.
   * @return a new parser.
   */
  public static TupleParser getTupleParser(ParticleContext ctx, String def)
  {
    TupleParser parser = _getContextMap().get(ctx);

    if ((parser == null) || !(parser.getDataType().contains(def)))
    {
      parser = create(def);
      _getContextMap().put(ctx, parser);
    }

    return parser;
  }
}
